<!--
Project: Single Sourcing Demo.
Component: Writer (File 3 of 3).
Filename: "finalize_modules.php".
Purpose: Loops through metadata file and writes out all documentation modules to a valid XML file using DOM.
-->

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
 <head>
  <title> Single Sourcing Demo: Finalize Modules </title>
 </head>

 <body>

<h1>Single Sourcing Demo</h1>
<h2>Step 3: Generate XML Document</h2>

<?php

/* 
The $bufferFilename variable is used to store the name of the temporary file we will be reading from. For this exercise, this file needs to be in the same directory as this script, but it could be changed by including the directory path as part of this variable.  The $xmlOutputFilename will be the name of the file used to store our XML documentation.
*/

$bufferFilename = "ss_buffer.txt";
$metaDataDirectory = "metadata/";
$xmlOutputFilename = "documentation.xml";

/* This function writes the beginning of the XML file using UTF-8 (Latin) encoding */
function startXMLdocument($xmlString) 
	{
	$xmlString = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
	$xmlString .= "<!-- XML Single Sourcing Documentation File -->\n";
	$xmlString .= "<!-- Automatically generated by Single Sourcing Demo -->\n";
	$xmlString .= "<!-- Created on ".date("MdY")." -->\n";
	$xmlString .= "<?xml-stylesheet type=\"text/xsl\" href=\"ss_allmodules_transform.xsl\"?>\n\n";
	return($xmlString);
	}

/* 
This function writes the document type definition (DTD) as well as the top level documentation container to the XML file 
*/
function insertDTD($xmlString) 
	{
	$xmlString .= "<!DOCTYPE documentation_modules [\n";
	$xmlString .= "\t<!ELEMENT documentation_modules (module)*>\n";
	$xmlString .= "\t<!ELEMENT module (id,feature,skill_level, documentation)>\n";
	$xmlString .= "\t<!ELEMENT id (#PCDATA)>\n";
	$xmlString .= "\t<!ELEMENT feature (#PCDATA)>\n";
	$xmlString .= "\t<!ELEMENT skill_level (#PCDATA)>\n";
	$xmlString .= "\t<!ELEMENT documentation (#PCDATA)>\n";
	$xmlString .= "]>\n\n";
	$xmlString .= "<documentation_modules>\n";
	return($xmlString);
	}

/* 
This function closes the documentation container 
*/
function endXMLdocument($xmlString)
	{
	$xmlString .= "</documentation_modules>\n";
	return($xmlString);
	}

/* 
This function will loop through the temporary buffer file and find each asset and its metadata.  It can tell where one asset ends and another begins because they are each separated by newlines (or the \n character sequence).  It can also tell where one element ends and the next one begins by the *** character sequence.  Each asset is converted into an XML element with its values written out into the XML file as character data.
*/
function addDocumentationCollection($filename,$xmlString)
	{
	if ($filecontents = file($filename))
		{
		$xmlString = startXMLdocument($xmlString);
		$xmlString = insertDTD($xmlString);
		foreach($filecontents as $singleLine)
			{
			$line = explode("\n",$singleLine);
			$units = explode("***",$line[0]);
			$xmlString .= "\t\t<module>\n";
			$xmlString .= "\t\t\t<id>".$units[0]."</id>\n";
			$xmlString .= "\t\t\t<feature>".$units[1]."</feature>\n";	
			$xmlString .= "\t\t\t<skill_level>".$units[2]."</skill_level>\n";
			$xmlString .= "\t\t\t<documentation>".$units[3]."</documentation>\n";
			$xmlString .= "\t\t</module>\n";			
			}
		$xmlString = endXMLdocument($xmlString);
		return($xmlString);
		}
	else
		{
		echo "Unable to open file!";
		}	
	}

/*
This function handles the actual writing of the XML document to a permanent file.  It will use the same metadata directory and filename as specified by the variables at the top of this script. 
*/
function writeXMLFile($metaDataDirectory, $fileName, $xmlString)
	{
	$fp = fopen($metaDataDirectory.$fileName, "w");
	if ($bytesWrittenToFile = fwrite($fp, $xmlString))
		{
		echo "XML file created successfully!<br />";
		return TRUE;
		}
	else
		{
		echo "There was a problem writing to file.<br />";
		echo "Here is the XML content: <br /><pre>$xmlString</pre>";
		return FALSE;
		}
	}

/*
Here is where the addDocumentationCollection is "called" to create the XML string that will be written to the permanent file.  For now, the entire XML document will be stored in a temporary string variable named $xml.
*/
$xml = addDocumentationCollection($bufferFilename, $xmlString);

/*
This segment of code writes the XML string to a permanent file.  If it is unable to create the file, the error message defined in the writeXMLFile function will be displayed.
*/
if (writeXMLFile($metaDataDirectory, $xmlOutputFilename, $xml))
	{
	echo "<br />";
	echo "Click here to <a href=\"".$metaDataDirectory.$xmlOutputFilename."\">view your documentation using an XSL transform</a>.<br />";
	echo "Click here to <a href=\"add_module.html\">begin a new documentation collection</a>.<br />";
	echo "Click here to <a href=\"parse_xml.php\">parse the documentation file using the single sourcing parser</a>.";
	}

?>

</body>
</html>

<!-- End of "finalize_modules.php" file -->