<!--
Project: Single Sourcing Demo.
Component: Reader/Parser (File 1 of 1).
Filename: "parse_xml.php".
Purpose: Creates an interactive browsing system for our documentation file based on user-selected skill level.
-->

<?php

/*
The $rootPath variable is used to indicate the full file path to the documentation files on your hard drive.  The $metaDataDirectory variable holds the value for the directory containing the XML files.
*/
$rootPath = "c:\\xampp\\htdocs\\xmlparsers\\ss\\";
$metaDataDirectory = "metadata\\";
$xmlOutputFilename = "documentation.xml";

/*
These two variables will be used to hold the skill level and module information once they have been selected by the user using the unordered (bulleted) list display on the page. 
*/
$selectedSkillLevel = $_GET["skill_level"];
$selectedModule = $_GET["module"];

/*
The purpose of this function is to list all skill levels currently available.  To add new skill levels, simply append their names to the end of the $skillLevels array.
*/
function listSkillLevels()
	{
	$skillLevels = array("Beginner","Intermediate","Advanced");
	echo "<p>Please select the skill level that best reflects your computing experience.</p>";
	echo "<ul>";
	foreach ($skillLevels as $sl)
		{
		echo "<li>";
		/* This creates a link such that when we click on it, the skill level value is passed along with it! */
		echo "<a href=\"parse_xml.php?skill_level={$sl}\">";
		echo $sl;
		echo "</a>";
		echo "</li>";
		}
	echo "</ul>";
	}

/*
The purpose of this function is to list all documentation features (topics) currently available.  Each topic must have all three skill level modules (beginner, intermediate, and advanced) available in order for this to work correctly.  To add new topics, simply append their names to the end of the $completedModules array (e.g. array("Introduction","Topic 2","Topic 3")...).
*/
function listTopics($skill)
	{
	$completedModules = array("Introduction");
	echo "<p>Please select a topic to browse.</p>";
	echo "<ul>";
	foreach ($completedModules as $module)
		{
		echo "<li>";
		/* This link will pass along both selected module and skill level information to the parser */
		echo "<a href=\"parse_xml.php?module={$module}&skill_level={$skill}\">";
		echo $module;
		echo "</a>";
		echo "</li>";
		}
	echo "</ul>";
	}
	
if (isset($selectedSkillLevel))
	{
	if (isset($selectedModule))
		{
		/* 
		This block of code will execute if BOTH skill level and module have been selected by the user.	For this example, we are creating a DOM parser. 
		*/
		$xml = new DOMDocument();

		/* Load the file using the path, directory, and XML filename specified at the top of this script */
		$xml->load($rootPath.$metaDataDirectory.$xmlOutputFilename);

		/* 
		With the DOM, we can now use our DTD to validate our XML document! 
		*/
		if ($result = @$xml->validate())
			{
			/* This block of code will "run" only if the XML document has successfully passed validation. */
			echo "<h1>".$selectedModule."</h1>";

			/* Using the DOM model, this will select all DOM nodes with the tag name "module" */
			$modules = $xml->getElementsByTagName("module");
			
			/* This will loop through each node in an iterative fashion. */			
			foreach ($modules as $module)
				{
				
				/* First, get any skill level nodes (there is actually only one) associated with each module */
				$skillLevels = $module->getElementsByTagName("skill_level");

				/* Now, get the value associated with that node.  This will be "beginner", "intermediate", or "advanced." */
				$skillLevel = $skillLevels->item(0)->nodeValue;
				
				/* This will check to see what value was contained in the node. */
				switch(strtolower($selectedSkillLevel))
					{
					/* If beginner skill level is selected, show only that skill level's content */
					case "beginner":
						if ($skillLevel == "beginner")
							{
							$node = $module->getElementsByTagName("documentation");
							$doc = $node->item(0)->nodeValue;
							echo "<p>".stripslashes($doc)."</p>";
							}
						break;
					/* If intermediate skill level is selected, show that skill level content plus beginning level content */
					case "intermediate":
						if ($skillLevel == "beginner" || $skillLevel == "intermediate")
							{
							$node = $module->getElementsByTagName("documentation");
							$doc = $node->item(0)->nodeValue;
							echo "<p>".stripslashes($doc)."</p>";
							}
						break;
					/* If advanced skill level is selected, show both prior skill levels plus advanced content */
					case "advanced":
					if ($skillLevel == "beginner" || $skillLevel == "intermediate" || $skillLevel == "advanced")
							{
							$node = $module->getElementsByTagName("documentation");
							$doc = $node->item(0)->nodeValue;
							echo "<p>".stripslashes($doc)."</p>";
							}
						break;												
					default:
						echo "<p>Error: unable to provide documentation for skill level $selectedSkillLevel.</p>";
						break;
					}
				}
			echo "<a href=\"parse_xml.php\">Back Home</a>";
			}
		else
			{
			echo "<p>Error: the documentation file was unable to validate against its internal DTD.  There is likely a problem with the XML documentation file.  Please contact technical support!</p>";
			}		
		}
	else
		{
		/* Show the list of available topics and embed the provided skill level into the links */
		listTopics($selectedSkillLevel);
		}
	}
else
	{
	/* Show the list of currently available skill levels */
	listSkillLevels();	
	}

?>

<!-- End of "parse_xml.php" file -->